/***************************************************************************
                          dcgui.cpp  -  description
                             -------------------
    begin                : Mon Okt  1 19:59:45 CEST 2001
    copyright            : (C) 2001 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qvbox.h>
#include <qaccel.h>
#include <qdockwindow.h>

// application specific includes
#include <stdlib.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include "dcgui.h"
#include "dcoptions.h"
#include "dcserverlist.h"
#include "dctransferview.h"
#include "dcconfig.h"
#include "dchubsearch.h"
#include "dcgeturl.h"
#include "dcspy.h"
#include "dcuserslist.h"
#include "dcfiletool.h"
#include "dctranslator.h"
#include "dcevent.h"
#include "dcpluginmanager.h"

#include "res/find.xpm"
#include "res/find_22x22.xpm"
#include "res/transfer.xpm"
#include "res/server.xpm"
#include "res/configure.xpm"
#include "res/configure_22x22.xpm"
#include "res/spy.xpm"
#include "res/exit.xpm"

#include <dclib/dcos.h>
#include <dclib/core/cbytearray.h>
#include <dclib/dcobject.h>
#include <dclib/cdownloadmanager.h>
#include <dclib/cutils.h>

DCGuiApp::DCGuiApp()
{
	StringMap * map;
	int width,height;

	pTransferView = 0;
	pServerList   = 0;
	pHubSearch    = 0;
	pSpy          = 0;
	pUsersList    = 0;

	setCaption( tr("DCGUI-QT " VERSION ) );

	SetTheme();

	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initView();
	initActions();
	initMenuBar();
	initToolBar();
	initStatusBar();

	if ( Config->GetBackgroundPixmap() != "" )
	{
		QPixmap bp;
		if ( bp.load(Config->GetBackgroundPixmap().Data()) )
			pWorkspace->setPaletteBackgroundPixmap(bp);
	}

	Config->GetAppGeometry( width, height );
	resize( width, height );

	// init transferview
	if ( Config->GetMap("TRANSFERVIEW",map) == TRUE )
	{
		if ( (*map)["DOCKED"].toInt() == 1 )
		{
			// resize if docked
			pTransferViewDock->dock();
			pTransferViewDock->setFixedExtentHeight((*map)["HEIGTH"].toInt());
			pTransferViewDock->setFixedExtentWidth((*map)["WIDTH"].toInt());
		}
		else
		{
			pTransferViewDock->undock();
			pTransferViewDock->resize( (*map)["WIDTH"].toInt(), (*map)["HEIGTH"].toInt() );
		}
	}

	viewToolBar->setOn(true);
	viewStatusBar->setOn(true);

	// init main window
	if ( Config->GetMap("MAINVIEW",map) == TRUE )
	{
		if ( (*map)["MAXIMIZED"].toInt() == 1 )
		{
			showMaximized();
		}
		else
		{
			show();
		}
	}

	if ( Config->GetToolBar() == FALSE )
		viewToolBar->toggle();
	if ( Config->GetStatusBar() == FALSE )
		viewStatusBar->toggle();
	if ( Config->GetShowTransferWin() == TRUE )
		pTransferViewDock->show();

	gDCTranslator = new DCTranslator();

	// install event filter on qApp ;-)
	qApp->installEventFilter(&m_ApplicationEvents);

	// init timer
	m_pTimer = new QTimer();

	connect( m_pTimer, SIGNAL(timeout()), this, SLOT(slotTimeout()) );

	// start timer
	m_pTimer->start( 1000 );
}

DCGuiApp::~DCGuiApp()
{
	// remove event-filter from appl.
	qApp->removeEventFilter(&m_ApplicationEvents);
	// top timer
	m_pTimer->stop();

	delete m_pTimer;

	if (pHubSearch)
	{
		pHubSearch->removeEventFilter(this);
		delete pHubSearch;
		pHubSearch = 0;
	}

	removeDockWindow(pTransferViewDock);
	pTransferView->removeEventFilter(this);

	if (pTransferViewDock)
	{
		pTransferViewDock->removeEventFilter(this);
		delete pTransferViewDock;
		pTransferViewDock = 0;
	}

	if (pTransferView)
	{
		//pTransferView->removeEventFilter(this);
		//delete pTransferView;
		pTransferView = 0;
	}

	if (pServerList)
	{
		pServerList->removeEventFilter(this);
		delete pServerList;
		pServerList = 0;
	}

	if (pSpy)
	{
		pSpy->removeEventFilter(this);
		delete pSpy;
		pSpy = 0;
	}

	if (pUsersList)
	{
		delete pUsersList;
		pUsersList = 0;
	}

	if ( gDCTranslator )
	{
		delete gDCTranslator;
		gDCTranslator = 0;
	}
}

void DCGuiApp::initActions()
{
	fileOptions = new QAction(tr("Options"), QIconSet( QPixmap( configure_xpm ) ), tr("&Options"), QAccel::stringToKey(tr("Ctrl+O")), this);
	fileOptions->setStatusTip(tr("Open option dialog"));
	fileOptions->setWhatsThis(tr(""));
	connect(fileOptions, SIGNAL(activated()), this, SLOT(slotFileOptions()));

	fileQuickOptions = new QAction(tr("Quick Options"), tr("Quick options"), QAccel::stringToKey(tr("F12")), this);
	fileQuickOptions->setStatusTip(tr("Context menu for a few options"));
	fileQuickOptions->setWhatsThis(tr(""));
	connect(fileQuickOptions, SIGNAL(activated()), this, SLOT(slotFileQuickOptions()));

	/** Default download mode */
	fileDefaultDldAsk = new QAction(tr("Download mode: Ask"), tr("Download mode: Ask"), QAccel::stringToKey(tr("")), this, 0, true);
//	fileDefaultDldAsk->setStatusTip(tr(""));
//	fileDefaultDldAsk->setWhatsThis(tr(""));
	fileDefaultDldSingle = new QAction(tr("Download mode: Single"), tr("Download mode: Single"), QAccel::stringToKey(tr("")), this, 0, true);
	fileDefaultDldMulti = new QAction(tr("Download mode: Multi"), tr("Download mode: Multi"), QAccel::stringToKey(tr("")), this, 0, true);
	fileDefaultDldGroup = new QActionGroup( this, "Download type", TRUE );
        connect( fileDefaultDldGroup, SIGNAL( selected(QAction*) ), this, SLOT(slotChangeDefaultDldMode(QAction*)) );
	fileDefaultDldGroup->add(fileDefaultDldAsk);
	fileDefaultDldGroup->add(fileDefaultDldSingle);
	fileDefaultDldGroup->add(fileDefaultDldMulti);
	switch (Config->GetDefaultDownloadMode())
	{
		case 0: fileDefaultDldAsk->setOn(TRUE); break;
		case 1: fileDefaultDldSingle->setOn(TRUE); break;
		case 2: fileDefaultDldMulti->setOn(TRUE); break;
	}

	fileReloadPlugins = new QAction(tr("Reload Plugins"), QIconSet( QPixmap("") ), tr("&Reload Plugins"), 0,  this);
	fileReloadPlugins->setStatusTip(tr("Reload Plugins"));
	fileReloadPlugins->setWhatsThis(tr(""));
	connect(fileReloadPlugins, SIGNAL(activated()), this, SLOT(slotFileReloadPlugins()));

	fileQuit = new QAction(tr("Exit"), QIconSet( QPixmap( exit_xpm ) ), tr("Exit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
	fileQuit->setStatusTip(tr("Quits the application"));
	fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
	connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

	viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
	viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
	viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
	connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

	viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
	viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
	viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
	connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

	windowCascade = new QAction(tr("Cascade"), tr("&Cascade"), 0, this);
	windowCascade->setStatusTip(tr("Cascades all windows"));
	windowCascade->setWhatsThis(tr("Cascade\n\nCascades all windows"));
	connect(windowCascade, SIGNAL(activated()), pWorkspace, SLOT(cascade()));

	windowTile = new QAction(tr("Tile"), tr("&Tile"), 0, this);
	windowTile->setStatusTip(tr("Tiles all windows"));
	windowTile->setWhatsThis(tr("Tile\n\nTiles all windows"));
	connect(windowTile, SIGNAL(activated()), pWorkspace, SLOT(tile()));

	windowTileHor = new QAction(tr("Tile &Horizontally"), tr("Tile &Horizontally"), 0, this);
	windowTileHor->setStatusTip(tr("Tiles all windows horizontally"));
	windowTileHor->setWhatsThis(tr("Tile Horizontally\n\nTiles all windows horizontally"));
	connect(windowTileHor, SIGNAL(activated()), this, SLOT(tileHorizontal()));

	windowMinimize = new QAction(tr("Minimize"), tr("&Minimize"), 0, this);
	windowMinimize->setStatusTip(tr("Minimize all windows"));
	windowMinimize->setWhatsThis(tr("Minimize\n\nMinimize all windows"));
	connect(windowMinimize, SIGNAL(activated()), this, SLOT(slotWindowMinimize()));

	windowCloseChats = new QAction(tr("Close chat windows"), tr("Close chat windows"), 0, this);
	windowCloseChats->setStatusTip(tr("Close all chat windows"));
	windowCloseChats->setWhatsThis(tr("Close chat windows\n\nClose all chat windows"));
	connect(windowCloseChats, SIGNAL(activated()), this, SLOT(slotWindowCloseChats()));

	windowCloseHubs = new QAction(tr("Close disconnected hubs"), tr("Close disconnected hubs"), 0, this);
	windowCloseHubs->setStatusTip(tr("Close all disconnected hub windows"));
	windowCloseHubs->setWhatsThis(tr("Close disconnected hubs\n\nClose all disconnected hub windows"));
	connect(windowCloseHubs, SIGNAL(activated()), this, SLOT(slotWindowCloseHubs()));

	windowAction = new QActionGroup(this, 0, false);
	windowAction->insert(windowCascade);
	windowAction->insert(windowTile);
	windowAction->insert(windowTileHor);
	windowAction->insert(windowMinimize);
	windowAction->insert(windowCloseChats);
	windowAction->insert(windowCloseHubs);

	/** server list */
	actionServerList = new QAction(tr("Hub List"), QIconSet( QPixmap( server_xpm ) ), tr("H&ub List"), QAccel::stringToKey(tr("Ctrl+U")), this, 0, false);
	actionServerList->setStatusTip(tr("Show Hub List"));
	actionServerList->setWhatsThis(tr(""));
	connect(actionServerList, SIGNAL(activated()), this, SLOT(slotActionServerList()));

	/** tranfer list */
	actionTransferList = new QAction(tr("Transfer List"), QIconSet( QPixmap( transfer_xpm ) ), tr("&Transfer List"), QAccel::stringToKey(tr("Ctrl+T")), this, 0, false);
	actionTransferList->setStatusTip(tr("Show Transfer List"));
	actionTransferList->setWhatsThis(tr(""));
	connect(actionTransferList, SIGNAL(activated()), this, SLOT(slotActionTransferList()));

	/** hub search */
	actionHubSearch = new QAction(tr("Hub Search"), QIconSet( QPixmap( find_xpm ) ), tr("&Hub Search"), QAccel::stringToKey(tr("Ctrl+H")), this, 0, false);
	actionHubSearch->setStatusTip(tr("Show Hub Search"));
	actionHubSearch->setWhatsThis(tr(""));
	connect(actionHubSearch, SIGNAL(activated()), this, SLOT(slotActionHubSearch()));

	/** spy */
	actionSpy = new QAction(tr("Spy"), QIconSet( QPixmap( spy_xpm ) ), tr("&Spy"), QAccel::stringToKey(tr("Ctrl+S")), this, 0, false);
	actionSpy->setStatusTip(tr("Show Hub Search"));
	actionSpy->setWhatsThis(tr(""));
	connect(actionSpy, SIGNAL(activated()), this, SLOT(slotActionSpy()));

	/** users */
	actionUsers = new QAction(tr("Users"), tr("Users"), QAccel::stringToKey(tr("Ctrl+F")), this, 0, false);
	actionUsers->setStatusTip(tr("Show users: search tab and friends tab"));
	actionUsers->setWhatsThis(tr(""));
	connect(actionUsers, SIGNAL(activated()), this, SLOT(slotActionUsers()));

	/** actiongroup for away mode */
	actionGroupAwayMode = new QActionGroup( this );
        actionGroupAwayMode->setExclusive( TRUE );

	/** away mode normal */
	actionAwayModeNormal = new QAction(tr("Normal"), tr("&Normal"), QAccel::stringToKey(tr("Ctrl+N")), actionGroupAwayMode, 0, true);
	actionAwayModeNormal->setStatusTip(tr("Set away mode"));
	actionAwayModeNormal->setWhatsThis(tr(""));
	actionAwayModeNormal->setToggleAction(TRUE);
	actionAwayModeNormal->setOn(TRUE);

	/** away mode away */
	actionAwayModeAway = new QAction(tr("Away"), tr("&Away"), QAccel::stringToKey(tr("Ctrl+A")), actionGroupAwayMode, 0, true);
	actionAwayModeAway->setStatusTip(tr("Set away mode"));
	actionAwayModeAway->setWhatsThis(tr(""));
	actionAwayModeAway->setToggleAction(TRUE);

        connect( actionGroupAwayMode, SIGNAL( selected( QAction* ) ), this, SLOT( slotChangeAwayMode() ) );

	helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
	helpAboutApp->setStatusTip(tr("About the application"));
	helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
	connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

	helpNewVersionApp = new QAction(tr("New Version"), tr("&New Version..."), 0, this);
	helpNewVersionApp->setStatusTip(tr("Check for new version"));
	helpNewVersionApp->setWhatsThis(tr("New Version\n\nCheck for new version"));
	connect(helpNewVersionApp, SIGNAL(activated()), this, SLOT(slotHelpNewVersion()));

}

void DCGuiApp::initMenuBar()
{
	///////////////////////////////////////////////////////////////////
	// MENUBAR

	///////////////////////////////////////////////////////////////////
	// menuBar entry pFileMenu
	pFileMenu=new QPopupMenu();
	fileOptions->addTo(pFileMenu);
	pQuickOptionsMenu=new QPopupMenu();
	fileDefaultDldGroup->addTo(pQuickOptionsMenu);
	pFileMenu->insertItem(tr("Quick Options"),pQuickOptionsMenu);
	pFileMenu->insertSeparator();
	fileReloadPlugins->addTo(pFileMenu);
	pFileMenu->insertSeparator();
	fileQuit->addTo(pFileMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar entry viewMenu
	pViewMenu=new QPopupMenu();
	pViewMenu->setCheckable(true);
	viewToolBar->addTo(pViewMenu);
	viewStatusBar->addTo(pViewMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar awayMenu
	pAwayMenu=new QPopupMenu();
	actionAwayModeNormal->addTo(pAwayMenu);
	actionAwayModeAway->addTo(pAwayMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar actionMenu
	pActionMenu=new QPopupMenu();
	actionServerList->addTo(pActionMenu);
	actionTransferList->addTo(pActionMenu);
	actionHubSearch->addTo(pActionMenu);
	actionSpy->addTo(pActionMenu);
	actionUsers->addTo(pActionMenu);
	pActionMenu->insertSeparator();
	pActionMenu->insertItem(tr("Away &Mode"),pAwayMenu);

	///////////////////////////////////////////////////////////////////
	// menuBar entry windowMenu
	pWindowMenu = new QPopupMenu(this);
	pWindowMenu->setCheckable(true);
	connect(pWindowMenu, SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

	///////////////////////////////////////////////////////////////////
	// menuBar entry helpMenu
	pHelpMenu=new QPopupMenu();
	helpAboutApp->addTo(pHelpMenu);
	pHelpMenu->insertSeparator();
	helpNewVersionApp->addTo(pHelpMenu);
	pHelpMenu->insertSeparator();
	pHelpMenu->insertItem(tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1);

	menuBar()->insertItem(tr("&File"), pFileMenu);
	menuBar()->insertItem(tr("&View"), pViewMenu);
	menuBar()->insertItem(tr("&Action"), pActionMenu);
	menuBar()->insertItem(tr("&Window"), pWindowMenu);
	menuBar()->insertItem(tr("&Help"), pHelpMenu);
}

void DCGuiApp::initToolBar()
{
	///////////////////////////////////////////////////////////////////
	// TOOLBAR
	fileToolbar = new QToolBar(this, tr("Action"));

	QWhatsThis::whatsThisButton(fileToolbar);

	fileToolbar->addSeparator();

	QPixmap preferencesIcon = QPixmap( configure_22x22_xpm );
//	QToolButton * preferencesOpen =
	new QToolButton( preferencesIcon, tr("Open option dialog"), QString::null,
				this, SLOT(slotFileOptions()), fileToolbar, "options" );

	fileToolbar->addSeparator();

	QPixmap serverIcon = QPixmap( server_xpm );
//	QToolButton * serverOpen =
	new QToolButton( serverIcon, tr("Show Hub List"), QString::null,
				this, SLOT(slotActionServerList()), fileToolbar, "hublist" );

	QPixmap searchIcon = QPixmap( find_22x22_xpm );
//	QToolButton * searchOpen =
	new QToolButton( searchIcon, tr("Show Hub Search"), QString::null,
				this, SLOT(slotActionHubSearch()), fileToolbar, "search" );

	QPixmap transferIcon = QPixmap( transfer_xpm );
//	QToolButton * transferOpen =
	new QToolButton( transferIcon, tr("Show Transfer List"), QString::null,
				this, SLOT(slotActionTransferList()), fileToolbar, "transfer list" );

	QPixmap spyIcon = QPixmap( spy_xpm );
//	QToolButton * spyOpen =
	new QToolButton( spyIcon, tr("Show Spy"), QString::null,
				this, SLOT(slotActionSpy()), fileToolbar, "spy" );

	// ADD USERS HERE

	fileToolbar->addSeparator();
}

void DCGuiApp::initStatusBar()
{
	///////////////////////////////////////////////////////////////////
	//STATUSBAR
	statusBar()->message(tr("Ready."));
}

void DCGuiApp::initView()
{
	QVBox* view_back = new QVBox( this );
	view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
	pWorkspace = new QWorkspace( view_back );
	setCentralWidget(view_back);

	setDockMenuEnabled(FALSE);

	// create serverlist view
	new DCServerList(pWorkspace,0,WDestructiveClose);
	pServerList->installEventFilter(this);
	pServerList->hide();
	pServerList->setEnabled(FALSE);

	// create dock window for transfer view
	pTransferViewDock = new QDockWindow( QDockWindow::InDock, this );
	pTransferViewDock->setResizeEnabled(TRUE);
	moveDockWindow(pTransferViewDock,Qt::DockBottom);
	pTransferViewDock->hide();

	// create transfer view
	new DCTransferView(pTransferViewDock,0,WDestructiveClose);
	pTransferView->SetWorkspace(pWorkspace);
	pTransferView->installEventFilter(this);

	pTransferViewDock->setWidget(pTransferView);

	// create multi hub search view
	new DCHubSearch(pWorkspace,0,WDestructiveClose);
	pHubSearch->installEventFilter(this);
	pHubSearch->hide();
	pHubSearch->setEnabled(FALSE);

	// create spy view
	new DCSpy(pWorkspace,0,WDestructiveClose);
	pSpy->installEventFilter(this);
	pSpy->hide();
	pSpy->setEnabled(FALSE);

	// create users view
	new DCUsersList(pWorkspace,0,WDestructiveClose);
	pUsersList->installEventFilter(this);
	pUsersList->hide();
	pUsersList->setEnabled(FALSE);

	statusDMLabel = new QLabel(statusBar(),"Label");
	statusDMLabel->setFrameShape( QFrame::NoFrame );
	statusDMLabel->setFrameShadow( QFrame::Plain );
	statusBar()->addWidget(statusDMLabel,0,TRUE);

	statusTRLabel = new QLabel(statusBar(),"Label");
	statusTRLabel->setFrameShape( QFrame::NoFrame );
	statusTRLabel->setFrameShadow( QFrame::Plain );
	statusBar()->addWidget(statusTRLabel,0,TRUE);

	statusFMLabel = new QLabel(statusBar(),"Label");
	statusFMLabel->setFrameShape( QFrame::NoFrame );
	statusFMLabel->setFrameShadow( QFrame::Plain );
	statusBar()->addWidget(statusFMLabel,0,TRUE);

	if ( Config->GetMode() == ecmActive )
	{
		CString error;
		if ( pTransferView->DLM_InitListen(error) == FALSE )
		{
			QString s;
			s = error.Data();

			if ( s == "" )
				s = tr("(no error message from socket)");
			else
				s = "(" + s + ")";

			QMessageBox::critical( 0, "DC-GUI",
			QString(tr("TCP listen failed, change to passive mode ! ") + s));

			Config->SetMode(ecmPassive);
		}
	}
}

/** setup all windows */
void DCGuiApp::initWindows()
{
	////////////////////////////////////////////////////////////////////
	StringMap * map;

	// setup hubview
	Config->GetMap("HUBVIEW",map);

	if ( (*map)["VISIBLE"].toInt() == 1 )
	{
		pServerList->setEnabled(TRUE);

		if ( (*map)["MAXIMIZED"].toInt() == 1 )
		{
			pServerList->showMaximized();
		}
		else
		{
			pServerList->show();
		}
	}
	else if ( (*map)["MINIMIZED"].toInt() == 1 )
	{
		pServerList->showMinimized();

		if ( (*map)["ENABLED"].toInt() == 0 )
			pServerList->hide();
		else
			pServerList->setEnabled(TRUE);
	}

	// setup searchview
	Config->GetMap("SEARCHVIEW",map);

	if ( (*map)["VISIBLE"].toInt() == 1 )
	{
		pHubSearch->setEnabled(TRUE);

		if ( (*map)["MAXIMIZED"].toInt() == 1 )
		{
			pHubSearch->showMaximized();
		}
		else
		{
			pHubSearch->show();
		}
	}
	else if ( (*map)["MINIMIZED"].toInt() == 1 )
	{
		pHubSearch->showMinimized();

		if ( (*map)["ENABLED"].toInt() == 0 )
			pHubSearch->hide();
		else
			pHubSearch->setEnabled(TRUE);
	}

	// setup spyview
	Config->GetMap("SPYVIEW",map);

	if ( (*map)["VISIBLE"].toInt() == 1 )
	{
		pSpy->setEnabled(TRUE);

		if ( (*map)["MAXIMIZED"].toInt() == 1 )
		{
			pSpy->showMaximized();
		}
		else
		{
			pSpy->show();
		}
	}
	else if ( (*map)["MINIMIZED"].toInt() == 1 )
	{
		pSpy->showMinimized();

		if ( (*map)["ENABLED"].toInt() == 0 )
			pSpy->hide();
		else
			pSpy->setEnabled(TRUE);
	}

	// setup userview
	Config->GetMap("USERVIEW",map);

	if ( (*map)["VISIBLE"].toInt() == 1 )
	{
		pUsersList->setEnabled(TRUE);

		if ( (*map)["MAXIMIZED"].toInt() == 1 )
		{
			pUsersList->showMaximized();
		}
		else
		{
			pUsersList->show();
		}
	}
	else if ( (*map)["MINIMIZED"].toInt() == 1 )
	{
		pUsersList->showMinimized();

		if ( (*map)["ENABLED"].toInt() == 0 )
			pUsersList->hide();
		else
			pUsersList->setEnabled(TRUE);
	}
}

/** */
bool DCGuiApp::queryExit()
{
	int exit=QMessageBox::information(this, tr("Quit..."),
				tr("Do you really want to quit?"),
				QMessageBox::Ok, QMessageBox::Cancel);

	if (exit==1)
	{
	}
	else
	{
	}

	return (exit==1);
}

/** */
void DCGuiApp::slotTimeout()
{
	if ( Config->GetAutoAwayMode() == TRUE )
	{
		// check for user input
		if ( m_ApplicationEvents.m_nNoUserInputTimer == 0 )
		{
			// check away mode and reset to normal
			if ( Config->GetAwayMode() != euamNORMAL )
			{
				actionAwayModeNormal->setOn(TRUE);
			}

			m_ApplicationEvents.m_nNoUserInputTimer = time(0);
		}
		else
		{
			// away timeout
			if ( (time(0)-m_ApplicationEvents.m_nNoUserInputTimer) >= Config->GetAutoAwayTime() )
			{
				// check away mode
				if ( Config->GetAwayMode() == euamNORMAL )
				{
					actionAwayModeAway->setOn(TRUE);
				}

				m_ApplicationEvents.m_nNoUserInputTimer = time(0);
			}
		}
	}
}

/** */
bool DCGuiApp::eventFilter(QObject* object, QEvent* event)
{
	QString s;

	if((event->type() == QEvent::Close)&&((DCGuiApp*)object!=this))
	{
		QCloseEvent* e=(QCloseEvent*)event;

		if ( ((DCTransferView*)object == pTransferView) && (pTransferView != 0) )
		{
			e->ignore();
			((DCTransferView*)object)->hide();
			((DCTransferView*)object)->setEnabled(FALSE);
		}
		else if ( ((DCServerList*)object == pServerList) && (pServerList != 0) )
		{
			e->ignore();
			pServerList->hide();
			pServerList->setEnabled(FALSE);
		}
		else if ( ((DCHubSearch*)object == pHubSearch) && (pHubSearch != 0) )
		{
			e->ignore();
			pHubSearch->hide();
			pHubSearch->setEnabled(FALSE);
		}
		else if ( ((DCSpy*)object == pSpy) && (pSpy != 0) )
		{
			e->ignore();
			pSpy->hide();
			pSpy->setEnabled(FALSE);
		}
		else if ( ((DCUsersList*)object == pUsersList) && (pUsersList != 0) )
		{
			e->ignore();
			pUsersList->hide();
			pUsersList->setEnabled(FALSE);
		}
		else
		{
			e->accept();
		}
	}
	else if((event->type() == QEvent::Close)&&((DCGuiApp*)object==this))
	{
		QCloseEvent* e=(QCloseEvent*)event;

		if ( slotFileQuit() == FALSE ) {
			e->ignore();
			return TRUE;
		}
	}
	else if( (event->type() == EVENT_UPDATE_DM_STATUSBAR) && ((DCGuiApp*)object!=this) )
	{
		DC_DownloadManagerEvent * e = (DC_DownloadManagerEvent*)event;

		s  = "SL: "+QString().setNum(e->m_pDownloadManagerInfo->slot_use_settings)+QString("/")+QString().setNum(e->m_pDownloadManagerInfo->slot_max) + " ";
		s += QString().setNum(e->m_pDownloadManagerInfo->slot_use_operator)+QString("/")+
		     QString().setNum(e->m_pDownloadManagerInfo->slot_use_user)+QString("/")+
		     QString().setNum(e->m_pDownloadManagerInfo->slot_use_special);

		s += QString(" UL: ")+CUtils::GetTransferString(e->m_pDownloadManagerInfo->Rate()).Data()+\
			QString(" DL: ")+\
			CUtils::GetTransferString(e->m_pDownloadManagerInfo->rate_dl).Data();

		statusDMLabel->setText(s);
	}
	else if( (event->type() == EVENT_UPDATE_FM_STATUSBAR) && ((DCGuiApp*)object!=this) )
	{
		DC_FileManagerEvent * e = (DC_FileManagerEvent*)event;

		if ( e->m_pFileManagerInfo->m_bCreateShareList )
		{
			s = "L ";
			s += QString().setNum(e->m_pFileManagerInfo->m_nShareListPercent,'f',2) + "%";
		}
		else if ( e->m_pFileManagerInfo->m_bCreateSearchIndex )
		{
			s = "S ";
			s += QString().setNum(e->m_pFileManagerInfo->m_nSearchIndexPercent,'f',2) + "%";
		}
		else
		{
			s = "";
		}

		statusFMLabel->setText(s);
	}
	else if( (event->type() == EVENT_UPDATE_TR_STATUSBAR) && ((DCGuiApp*)object!=this) )
	{
		DC_TrafficInfoEvent * e = (DC_TrafficInfoEvent*)event;

		s = ( "RX: " + CUtils::GetSizeString(e->m_pTrafficInfo->m_nRx,Config->GetUnit()) + " TX: " +
		    CUtils::GetSizeString(e->m_pTrafficInfo->m_nTx,Config->GetUnit()) ).Data();

		statusTRLabel->setText(s);
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void DCGuiApp::tileHorizontal()
{
	int i,count =0;
	int y = 0;
	int heightForEach;

	// primitive horizontal tiling
	QWidgetList windows = pWorkspace->windowList();
	if ( !windows.count() )
		return;

	for ( i = 0; i < int(windows.count()); ++i )
	{
		if ( windows.at(i)->isEnabled() )
			count++;
	}

	if ( count > 0 )
	{
		heightForEach = pWorkspace->height() / count;

		for ( i = 0; i < int(windows.count()); ++i )
		{
			if ( !windows.at(i)->isEnabled() )
				continue;
			QWidget *window = windows.at(i);
			if ( (window->testWState( WState_Maximized )) || (window->isMinimized()))
			{
				// prevent flicker
				window->hide();
				window->showNormal();
			}

			int preferredHeight = window->minimumHeight()+window->parentWidget()->baseSize().height();
			int actHeight = QMAX(heightForEach, preferredHeight);

			window->parentWidget()->setGeometry( 0, y, pWorkspace->width(), actHeight );
			y += actHeight;
		}
	}
}

/** minimize all windows */
void DCGuiApp::slotWindowMinimize()
{
	uint i;

	for ( i = 0; i < pWorkspace->windowList().count(); i++ )
	{
		if ( pWorkspace->windowList().at(i)->isVisible() )
			pWorkspace->windowList().at(i)->showMinimized();
	}
}

/** close all chat windows */
void DCGuiApp::slotWindowCloseChats()
{
	pServerList->CloseAllChats();
}

/** close all chat windows */
void DCGuiApp::slotWindowCloseHubs()
{
	pServerList->CloseDisconnectedHubs();
}

/** */
void DCGuiApp::slotFileOptions()
{
	DCOptions * od;

	statusBar()->message(tr("Open options dialog..."));

	od = new DCOptions();

	od->SetValues();

	if ( QDialog::Accepted == od->exec() )
	{
		// move values to dcconfig
		od->GetValues();
		// save values
		Config->Save();
		// update connected servers
                pServerList->SendMyInfoToConnectedServers();
	}

	delete od;

	/** reset or set theme */
	SetTheme();

	statusBar()->message(tr("Ready."));
}

/** */
void DCGuiApp::slotFileReloadPlugins()
{
	pPluginManager->DeInitPlugins();
	pPluginManager->Load();
}

/** */
void DCGuiApp::slotFileQuickOptions()
{
	pQuickOptionsMenu->exec(QCursor::pos());
}

/** */
void DCGuiApp::slotChangeDefaultDldMode(QAction* DldType)
{
	DldType->setOn(TRUE);
	if (fileDefaultDldAsk->isOn())
	{
		Config->SetDefaultDownloadMode(0);
	}
	else if (fileDefaultDldSingle->isOn())
	{
		Config->SetDefaultDownloadMode(1);
	}
	else  if (fileDefaultDldMulti->isOn())
	{
		Config->SetDefaultDownloadMode(2);
	}
}

bool DCGuiApp::slotFileQuit()
{ 
	StringMap * map;

	statusBar()->message(tr("Exiting application..."));

	if ( Config->GetQueryOnExit() )
	{
		if ( queryExit() == false )
		{
			statusBar()->message(tr("Exiting application... aborted"));
			return FALSE;
		}
	}

	pTransferView->DLM_Shutdown();

	while( pTransferView->DLM_ShutdownState() != essSHUTDOWNREADY )
	{
		qApp->processEvents();
#ifdef WIN32
		Sleep(100);
#else
		usleep(100);
#endif
	}

	Config->SetToolBar(viewToolBar->isOn());
	Config->SetStatusBar(viewStatusBar->isOn());
	Config->SetAppGeometry( width(), height() );

	// save transfer view settings
	Config->GetMap("TRANSFERVIEW",map);

	(*map)["X"]      = QString().setNum(pTransferViewDock->x());
	(*map)["Y"]      = QString().setNum(pTransferViewDock->y());
	(*map)["WIDTH"]  = QString().setNum(pTransferViewDock->width());
	(*map)["HEIGTH"] = QString().setNum(pTransferViewDock->height());
	(*map)["DOCKED"] = QString().setNum((pTransferViewDock->area()!=0));

	Config->SetShowTransferWin( pTransferViewDock->isVisible() );

	// save transfer list settings
	pTransferView->DeInitDocument();

	// save hub list settings
	pServerList->DeInitDocument();

	// save search view settings
	pHubSearch->DeInitDocument();

	// save spy view settings
	pSpy->DeInitDocument();

	// save user view settings
	pUsersList->DeInitDocument();

	// init main window
	Config->GetMap("MAINVIEW",map);
	(*map)["MAXIMIZED"] = QString().setNum(isMaximized());

	// save gui config
	Config->SaveDCGui();

	qApp->quit();

	statusBar()->message(tr("Ready."));

	return TRUE;
}

void DCGuiApp::slotViewToolBar(bool toggle)
{
	statusBar()->message(tr("Toggle toolbar..."));
	///////////////////////////////////////////////////////////////////
	// turn Toolbar on or off
	if (toggle== false)
	{
		fileToolbar->hide();
	}
	else
	{
		fileToolbar->show();
	};

	statusBar()->message(tr("Ready."));
}

void DCGuiApp::slotViewStatusBar(bool toggle)
{
	statusBar()->message(tr("Toggle statusbar..."));
	///////////////////////////////////////////////////////////////////
	//turn Statusbar on or off
  
	if (toggle == false)
	{
		statusBar()->hide();
	}
	else
	{
		statusBar()->show();
	}

	statusBar()->message(tr("Ready."));
}

/** */
void DCGuiApp::slotHelpAbout()
{
	QMessageBox::about(this,tr("About..."),
                     tr("DCGUI-QT\nVersion " VERSION
                     "\n(c) 2001-2003 by Mathias Kster"
		     "\nhttp://dc.ketelhot.de"));
}

/** check for new versions */
void DCGuiApp::slotHelpNewVersion()
{
	QString release="";
	DCGetURL * d;

	d = new DCGetURL("http://dc.ketelhot.de/version.xml");

	if ( d->Get() == QDialog::Accepted )
	{
		CString s;
		CByteArray in;
		d->GetData(&in);
		s.Set((char*)in.Data(),in.Size());

		if ( s != "" )
		{
			release = Config->ParseVersion(s).Data();
		}

		if ( release == "" )
		{
			release = tr("Can't get version info from server");
		}
		else
		{
			release = tr("Current version is: ") + release;
		}

		QMessageBox::about(this,tr("New version..."),
				release );
	}

	delete d;
}

/** */
void DCGuiApp::slotStatusHelpMsg(const QString &text)
{
	///////////////////////////////////////////////////////////////////
	// change status message of whole statusbar temporary (text, msec)
	statusBar()->message(text, 2000);
}

/** change user away mode */
void DCGuiApp::slotChangeAwayMode()
{
	switch(Config->GetAwayMode())
	{
		case euamAWAY:
			actionAwayModeNormal->setOn(TRUE);
			Config->SetAwayMode(euamNORMAL);
			break;
		default:
			actionAwayModeAway->setOn(TRUE);
			Config->SetAwayMode(euamAWAY);
			break;
	}

	pServerList->SendMyInfoToConnectedServers();
}


/** */
void DCGuiApp::windowMenuAboutToShow()
{
	int c;

	pWindowMenu->clear();	
	windowCascade->addTo(pWindowMenu);
	windowTile->addTo(pWindowMenu);
	windowTileHor->addTo(pWindowMenu);
	windowMinimize->addTo(pWindowMenu);
	windowCloseChats->addTo(pWindowMenu);
	windowCloseHubs->addTo(pWindowMenu);
	
	if ( pWorkspace->windowList().isEmpty() )
	{
		windowAction->setEnabled(false);
	}
	else
	{
		windowAction->setEnabled(true);
	}

	pWindowMenu->insertSeparator();

	QWidgetList windows = pWorkspace->windowList();

	c = 0;

	for ( int i = 0; i < int(windows.count()); ++i )
	{
		if ( windows.at(i)->isEnabled() )
		{
			int id = pWindowMenu->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
			pWindowMenu->setItemParameter( id, i );
			pWindowMenu->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
			c++;
		}
	}

	if ( c == 0 )
	{
		windowAction->setEnabled(false);
	}
}

/** */
void DCGuiApp::windowMenuActivated( int id )
{
	QWidget* w = pWorkspace->windowList().at( id );
	if ( w )
	{
		w->show();
		w->setFocus();
	}
}

/** show/hide transfer list window */
void DCGuiApp::slotActionTransferList()
{
	statusBar()->message(tr("Show/hide transfer list window ..."));

	if ( pTransferView != 0 )
	{
		if ( pTransferView->isVisible() )
		{
			pTransferViewDock->setEnabled(FALSE);
			pTransferViewDock->hide();
		}
		else
		{
			pTransferViewDock->setEnabled(TRUE);
			pTransferViewDock->show();
		}
	}

	statusBar()->message(tr("Ready."));
}

/** show/hide server list window */
void DCGuiApp::slotActionServerList()
{
	statusBar()->message(tr("Show/hide hub list window ..."));

	if  ( pServerList != 0 )
	{
		if ( pServerList->isMinimized() )
		{
			pServerList->setEnabled(TRUE);
			pServerList->showNormal();
		}
		else if ( pServerList->isVisible() )
		{
			pServerList->setEnabled(FALSE);
			pServerList->hide();
		}
		else if ( pServerList->isMaximized() )
		{
			pServerList->setEnabled(TRUE);
			pServerList->showMaximized();
		}
		else
		{
			pServerList->setEnabled(TRUE);
			pServerList->show();
		}
	}

	statusBar()->message(tr("Ready."));
}

/** show/hide multi hub search window */
void DCGuiApp::slotActionHubSearch()
{
	statusBar()->message(tr("Show/hide hub search window ..."));

	if ( pHubSearch != 0 )
	{
		if ( pHubSearch->isMinimized() )
		{
			pHubSearch->setEnabled(TRUE);
			pHubSearch->showNormal();
		}
		else if ( pHubSearch->isVisible() )
		{
			pHubSearch->setEnabled(FALSE);
			pHubSearch->hide();
		}
		else if ( pHubSearch->isMaximized() )
		{
			pHubSearch->setEnabled(TRUE);
			pHubSearch->showMaximized();
		}
		else
		{
			pHubSearch->setEnabled(TRUE);
			pHubSearch->show();
		}
	}

	statusBar()->message(tr("Ready."));
}

/** show/hide spy window */
void DCGuiApp::slotActionSpy()
{
	statusBar()->message(tr("Show/hide spy window ..."));

	if ( pSpy != 0 )
	{
		if ( pSpy->isMinimized() )
		{
			pSpy->setEnabled(TRUE);
			pSpy->showNormal();
		}
		else if ( pSpy->isVisible() )
		{
			pSpy->setEnabled(FALSE);
			pSpy->hide();
		}
		else if ( pSpy->isMaximized() )
		{
			pSpy->setEnabled(TRUE);
			pSpy->showMaximized();
		}
		else
		{
			pSpy->setEnabled(TRUE);
			pSpy->show();
		}
	}

	statusBar()->message(tr("Ready."));
}

/** show/hide users window */
void DCGuiApp::slotActionUsers()
{
	statusBar()->message(tr("Show/hide users window ..."));

	if ( pUsersList != 0 )
	{
		if ( pUsersList->isMinimized() )
		{
			pUsersList->setEnabled(TRUE);
			pUsersList->showNormal();
		}
		else if ( pUsersList->isVisible() )
		{
			pUsersList->setEnabled(FALSE);
			pUsersList->hide();
		}
		else if ( pUsersList->isMaximized() )
		{
			pUsersList->setEnabled(TRUE);
			pUsersList->showMaximized();
		}
		else
		{
			pUsersList->setEnabled(TRUE);
			pUsersList->show();
		}
	}

	statusBar()->message(tr("Ready."));
}

/** */
void DCGuiApp::SetTheme()
{
	QString s;

	if ( Config->GetThemeSupport() == TRUE )
	{
		s = Config->GetTheme().Data();

		if ( s == "" )
		{
			printf("Can't set empty theme");
		}
		else
		{
			QApplication::setStyle( s );
		}
	}
}
