/***************************************************************************
                        dcuserslist.cpp  -  description
                             -------------------
    begin                : Thu Nov 21 2002
    copyright            : (C) 2002 by Franois Gannaz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qlistview.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include <dcconfig.h>
#include <dcevent.h>
#include <dcserverlist.h>
#include <dcmenuhandler.h>

#include "dcuserslist.h"

DCUsersList * pUsersList = 0;

/** */
DCUsersList::DCUsersList(QWidget* parent, const char *name, int wflags) : DCDialogUsersList(parent, name, wflags)
{
	// set default icon
	//	setIcon( QPixmap( ??? ) );

	pWorkspace = (QWorkspace*)parent;

	InitDocument();

	pUsersList = this;
}

/** */
DCUsersList::~DCUsersList()
{
}

/** */
int DCUsersList::selectedItems( QListView * list, QPtrList<QListViewItem> & lst )
{
	QListViewItemIterator it( (QListView *)list );

	for ( ; it.current(); it++ )
	{
		if ( it.current()->isSelected() )
		{
			lst.append(it.current());
		}
	}

	return lst.count();
}

/** */
void DCUsersList::InitDocument()
{
	StringMap * map;

	// restore settings
	if ( Config->GetMap("USERVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGTH"].toInt() > 0) )
		{
			setGeometry( (*map)["X"].toInt(), (*map)["Y"].toInt(), (*map)["WIDTH"].toInt(), (*map)["HEIGTH"].toInt() );
		}
	}
	
	connect( ListView_FRIENDS, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedFriendList(QListViewItem*, const QPoint &, int )) );

	// friends list
	m_pFriendList = new CStringList();

	// load frienlist
	Config->LoadDCFriendList( m_pFriendList );

	ShowFriendsList();

	installEventFilter(this);
}

/** */
void DCUsersList::DeInitDocument()
{
	StringMap * map;

	// save search view settings
	Config->GetMap("USERVIEW",map);

	(*map)["X"]         = QString().setNum(x());
	(*map)["Y"]         = QString().setNum(y());
	(*map)["WIDTH"]     = QString().setNum(width());
	(*map)["HEIGTH"]    = QString().setNum(height());
	(*map)["VISIBLE"]   = QString().setNum(isVisible());
	(*map)["MAXIMIZED"] = QString().setNum(isMaximized());
	(*map)["MINIMIZED"] = QString().setNum(isMinimized());
	(*map)["ENABLED"]   = QString().setNum(isEnabled());
}

/** event filter */
bool DCUsersList::eventFilter( QObject * object, QEvent * event )
{
	DCFriendObject * FriendObject;

	if( event->type() == EVENT_UPDATE_FRIEND )
	{
		DC_FriendEvent * e = (DC_FriendEvent*)event;

		if ( (e->m_pFriendObject->m_sName == "") && (pServerList) )
		{
			// check all friends online state
			FriendObject = 0;

			while( m_pFriendList->Next((CObject *&)FriendObject) == 1 )
			{
				if ( pServerList->IsUserOnline( FriendObject->m_sName, "", 0 ) == TRUE )
				{
					FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER));
				}
				else
				{
					FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER_AWAY));
				}
			}
		}
		else if ( m_pFriendList->Get(e->m_pFriendObject->m_sName,(CObject*&)FriendObject) == 0 )
		{
			if ( e->m_pFriendObject->m_eAwayMode != FriendObject->m_eAwayMode )
			{
				// update user
				FriendObject->m_eAwayMode = e->m_pFriendObject->m_eAwayMode;

				switch(FriendObject->m_eAwayMode)
				{
					case euamOFFLINE:
						FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER_AWAY));
						break;
					default:
						FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER));
						break;
				}
			}
		}
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** */
void DCUsersList::ShowFriendsList()
{
	DCFriendObject * FriendObject;

	FriendObject = 0;

	while( m_pFriendList->Next((CObject *&)FriendObject) == 1 )
	{
		FriendObject->m_pItem = new QListViewItem( ListView_FRIENDS, FriendObject->m_sName.Data() );
		FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER_AWAY));
	}
}

/** */
void DCUsersList::DelFriend( CString name )
{
	DCFriendObject * FriendObject;

	if ( m_pFriendList->Get(name,(CObject*&)FriendObject) == 0 )
	{
		ListView_FRIENDS->takeItem(FriendObject->m_pItem);
		delete FriendObject->m_pItem;

		// remove from list
		m_pFriendList->Del(name);

		// save list
		Config->SaveDCFriendList( m_pFriendList );
	}
}

/** */
void DCUsersList::AddFriend( CString name, CString hubname, CString hubhost, CString description )
{
	DCFriendObject * FriendObject;

	if ( m_pFriendList->Get(name,(CObject*&)FriendObject) != 0 )
	{
		FriendObject = new DCFriendObject();

		FriendObject->m_sName        = name;
		FriendObject->m_sHubName     = hubname;
		FriendObject->m_sHubHost     = hubhost;
		FriendObject->m_sDescription = description;

		FriendObject->m_pItem = new QListViewItem( ListView_FRIENDS, FriendObject->m_sName.Data() );

		m_pFriendList->Add(name,FriendObject);

		if ( pServerList )
		{
			if ( pServerList->IsUserOnline( FriendObject->m_sName, "", 0 ) == TRUE )
			{
				FriendObject->m_eAwayMode = euamONLINE;
			}
		}

		switch(FriendObject->m_eAwayMode)
		{
			case euamOFFLINE:
				FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER_AWAY));
				break;
			default:
				FriendObject->m_pItem->setPixmap(0,Config->GetUserIcon(euitUSER));
				break;
		}

		// save friendlist
		Config->SaveDCFriendList( m_pFriendList );
	}
}

/** */
void DCUsersList::UpdateFriend( CString /*name*/, CString /*host*/, CString /*description*/ )
{
}

/** */
void DCUsersList::slotRightButtonClickedFriendList(QListViewItem*, const QPoint &, int )
{
	int id;
	QPopupMenu *m;
	QPtrList<QListViewItem> selitems;
	QListViewItem * curitem;

	if ( selectedItems(ListView_FRIENDS,selitems) == 0 )
	{
		return;
	}

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiREMOVE );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == -1 )
	{
		return;
	}

	if ( id == emiREMOVE )
	{
	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			DelFriend(curitem->text(0).ascii());
		}
	}
}
